package xyz.leomurca.csgomatches.data.mapper

import xyz.leomurca.csgomatches.data.model.MatchDto
import xyz.leomurca.csgomatches.data.model.TeamDetailsDto
import xyz.leomurca.csgomatches.domain.model.League
import xyz.leomurca.csgomatches.domain.model.Match
import xyz.leomurca.csgomatches.domain.model.MatchStatus
import xyz.leomurca.csgomatches.domain.model.Opponent
import xyz.leomurca.csgomatches.domain.model.Player
import xyz.leomurca.csgomatches.domain.model.Serie
import xyz.leomurca.csgomatches.domain.model.Team

fun MatchDto.toDomain(): Match {
    return Match(
        beginAt = beginAt,
        opponents = opponents.map { op ->
            Opponent(
                id = op.opponent.id,
                name = op.opponent.name ?: "",
                imageUrl = op.opponent.imageUrl ?: ""
            )
        },
        league = League(
            id = league.id,
            name = league.name ?: "",
            imageUrl = league.imageUrl ?: ""
        ),
        serie = Serie(
            id = serie.id,
            name = serie.fullName ?: ""
        ),
        matchStatus = status?.toMatchStatus() ?: MatchStatus.UNKNOWN
    )
}

private fun String?.toMatchStatus() =
    if (this == "running") MatchStatus.LIVE else MatchStatus.SCHEDULED

fun TeamDetailsDto.toDomain(): Team {
    return Team(
        id = id,
        name = name,
        imageUrl = imageUrl,
        players = players.map {
            Player(
                id = it.id,
                nickName = it.name,
                firstName = it.firstName,
                lastName = it.lastName,
                imageUrl = it.imageUrl
            )
        }
    )
}