package xyz.leomurca.csgomatches.ui.screens.matches

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.launch
import xyz.leomurca.csgomatches.domain.model.Match
import xyz.leomurca.csgomatches.domain.model.Resource
import xyz.leomurca.csgomatches.domain.repository.MatchRepository
import javax.inject.Inject

@HiltViewModel
class MatchesViewModel @Inject constructor(
    private val matchRepository: MatchRepository
) : ViewModel() {

    private val _uiState = MutableStateFlow<MatchesUiState>(MatchesUiState.Loading)
    val uiState: StateFlow<MatchesUiState> = _uiState.asStateFlow()

    private var currentPage = 1
    private var isLoadingNextPage = false

    init {
        loadUpcomingMatches()
    }

    fun loadUpcomingMatches() {
        currentPage = 1
        viewModelScope.launch {
            _uiState.value = MatchesUiState.Loading
            _uiState.value = when (val result = matchRepository.upcomingMatches(currentPage)) {
                is Resource.Success -> MatchesUiState.Success(result.data)
                is Resource.Error -> MatchesUiState.Error(result.message)
            }
        }
    }

    fun loadNextPage() {
        val current = _uiState.value as? MatchesUiState.Success ?: return
        if (isLoadingNextPage || current.endReached) return

        isLoadingNextPage = true
        currentPage++

        viewModelScope.launch {
            _uiState.value = current.copy(isPaginating = true)

            val result = matchRepository.upcomingMatches(currentPage)
            val newMatches = (result as? Resource.Success)?.data.orEmpty()

            _uiState.value = current.copy(
                matches = current.matches + newMatches,
                isPaginating = false,
                endReached = newMatches.size < 20
            )

            isLoadingNextPage = false
        }
    }


    sealed class MatchesUiState {
        data class Success(
            val matches: List<Match>,
            val isPaginating: Boolean = false,
            val endReached: Boolean = false
        ) : MatchesUiState()

        data class Error(val message: String) : MatchesUiState()
        object Loading : MatchesUiState()
    }
}